/**************************************************************************
* Primo esempio:                                                          *
* crea un task e ci comunica mediante porte e messaggi                    *
**************************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <exec/tasks.h>
#include <exec/ports.h>
#include <clib/exec_protos.h>
#include <clib/alib_protos.h>
#include <stdio.h>

/* definizione comandi */
#define COMANDO1 0L
#define COMANDO2 1L
#define ESCI     2L

/* nome task */
#define NOMETASK "esempio1_task"

/* grandezza stack */
#define GRANDSTACK 20000

/* struttura del nostro messaggio */
struct Messaggio
{
  struct Message Msg;
  ULONG Comando,Dato;
};

/* variabili globali */
struct MsgPort *porta,*porta2;
struct Task *nuovotask;

/* il nostro task, apre la porta per la comunicazione, attende i messaggi e
   li stampa a video; quando il messaggio e' ESCI rimuove la porta e attende
   di venire eliminato dal programma */
void task()
{
  register ULONG ex = 0;
  struct Messaggio *mm;
  
  porta = (struct MsgPort *)CreatePort("esempio1_portaric",0);
  if (porta == NULL)
  {
    printf("Non posso creare la porta\n");
    Wait(0L);
  }

  while(ex == 0)
  {
    if ((mm = (struct Messaggio *)GetMsg(porta)) == NULL)
    {
      WaitPort(porta);
      continue;
    }
    printf("Comando - %ld; dato - %ld\n",mm->Comando,mm->Dato);
    if (mm->Comando == ESCI) ex = 1;
    ReplyMsg((struct Message *)mm);
  }
  DeletePort(porta);
  Wait(0L);
}

/* ecco la procedura per spedire un messaggio cosi' come e' definita sul
   ROM Kernel Manual che e' assicurata contro ogni imprevisto.
   La funzione ritorna TRUE se ha potuto spedire il messaggio FALSE altrimenti */
BOOL SafePutToPort(struct Message *message, STRPTR portname)
{
  struct MsgPort *port;
  
  Forbid();
  port = (struct MsgPort *)FindPort(portname);
  if (port) PutMsg(port,message);
  Permit();
  return(port ? TRUE : FALSE);
}

void main()
{
  struct Messaggio Mess;
  
  /* creazione porta per la trasmissione */
  porta2 = (struct MsgPort *)CreatePort("esempio1_portatra",0);
  if (porta2 == NULL)
  {
    printf("Non posso creare la porta\n");
    exit(0);
  }

  /* creazione task */
  nuovotask = (struct Task *)CreateTask(NOMETASK,0,task,GRANDSTACK);
  if (task == NULL)
  {
    DeletePort(porta2);
    printf("Non posso creare il task\n");
    exit(0);
  }

  /* Attendiamo che il task faccia le dovute inizializzazioni */
  Delay(20);
  /* preparazione messaggio */
  Mess.Msg.mn_Node.ln_Type = NT_MESSAGE;
  Mess.Msg.mn_Length = sizeof(struct Messaggio);
  Mess.Msg.mn_ReplyPort = porta2;
  Mess.Comando = COMANDO1;
  Mess.Dato = 100;
  /* invio messaggio */
  if (SafePutToPort((struct Message *)(&Mess),"esempio1_portaric") == FALSE)
    printf("Messaggio non spedito\n");

  Delay(20);
  Mess.Comando = ESCI;
  Mess.Dato = 200;
  if (SafePutToPort((struct Message *)(&Mess),"esempio1_portaric") == FALSE)
    printf("Messaggio non spedito\n");

  /* aspettiamo che il task liberi tutte le risorse precedentemente occupate */
  Delay(20);
  Forbid();
  DeletePort(porta2);
  DeleteTask(nuovotask);
  Permit();
}
